import pandas as pd
import fastavro
import copy
import json
from fastavro import writer, parse_schema

# Leemos el csv mediante pandas
df = pd.read_csv('ventas.csv',sep=';')

# Definimos el esquema
schema = {
    'name': 'Ventas',
    'namespace' : 'Ventas',
    'type': 'record',
    'fields': [
        {'name': 'ProductID', 'type': 'int'},
        {'name': 'Date', 'type': 'string'},
        {'name': 'Zip', 'type': 'string'},
        {'name': 'Units', 'type': 'int'},
        {'name': 'Revenue', 'type': 'float'},
        {'name': 'Country', 'type': 'string'}
    ]
}
schemaParseado = parse_schema(schema)

# Convertimos el Dataframe a una lista de diccionarios
records = df.to_dict('records')

# Persistimos en un fichero avro
with open('ventas.avro', 'wb') as f:
    writer(f, schemaParseado, records,'deflate')
    
# abrimos el archivo creado, lo leemos y mostramos línea a línea
with open("ventas.avro", "rb") as f:
    reader = fastavro.reader(f)
    # copiamos los metadatos del fichero leído
    metadata = copy.deepcopy(reader.metadata)
    # obtenemos el schema del fichero leído
    schemaReader = copy.deepcopy(reader.writer_schema)
    schemaFromFile = json.loads(metadata['avro.schema'])
    # Lee los primeros 10 registros
    primeros_10_ventas = [venta for _, venta in zip(range(10), reader)]

print(f'Schema del fichero ventas.avro:\n {schemaFromFile}')
print(f'Ventas:\n {primeros_10_ventas}')

