from pyspark import SparkContext
from pyspark.streaming import StreamingContext

# Crear un SparkContext y un StreamingContext
sc = SparkContext("local[2]", "NetworkWordCount")
ssc = StreamingContext(sc, 1)  # El intervalo de lote es de 1 segundo

# Configurar el punto de control (checkpoint) para mantener la información de estado
ssc.checkpoint("checkpoint")

# Crear un DStream que se conecte al puerto 8989 de localhost
lines = ssc.socketTextStream("localhost", 8989)

# Dividir las líneas en palabras
words = lines.flatMap(lambda line: line.split(" "))

# Mapear cada palabra a un par (palabra, 1)
pairs = words.map(lambda word: (word, 1))

# Agregar los pares por palabra utilizando una ventana de 10 segundos, deslizándose cada 5 segundos
windowed_word_counts = pairs.reduceByKeyAndWindow(lambda a, b: a + b, windowDuration=10, slideDuration=5)

# Imprimir los resultados
windowed_word_counts.pprint()

# Iniciar el StreamingContext
ssc.start()

# Esperar la finalización
ssc.awaitTermination()

