from pyspark import SparkContext
from pyspark.streaming import StreamingContext
from pyspark.streaming.kafka import KafkaUtils

# Crear un SparkContext y un StreamingContext
sc = SparkContext(appName="KafkaSparkStreaming")
ssc = StreamingContext(sc, 5)  # Intervalo de 5 segundos

# Conectarse a Kafka y leer el flujo de datos del tópico 'test'
kafkaStream = KafkaUtils.createDirectStream(ssc, ["test"], {"metadata.broker.list": "localhost:9092"})

# Procesamiento de los datos: Contar las palabras
lines = kafkaStream.map(lambda x: x[1])  # x[1] es el valor del mensaje
words = lines.flatMap(lambda line: line.split(" "))
word_counts = words.map(lambda word: (word, 1)).reduceByKey(lambda a, b: a + b)

# Imprimir los resultados en la consola
word_counts.pprint()

# Iniciar el StreamingContext
ssc.start()
ssc.awaitTermination()

