from airflow.models import DAG
from datetime import datetime
import json
import pandas as pd
import requests

from airflow.providers.http.sensors.http import HttpSensor
from airflow.providers.http.operators.http import SimpleHttpOperator
from airflow.providers.sqlite.operators.sqlite import SqliteOperator
from airflow.operators.python import PythonOperator
from airflow.operators.bash import BashOperator

default_args = { "start_date":datetime(2024,1,1) }

def get_data(ti):
	response=requests.get('https://randomuser.me/api')
	data=json.loads(response.text)
	ti.xcom_push(key="data", value=data)
	return data
    
def _procesa_usuario(ti):
    #usuarios=ti.xcom_pull(task_ids=['extraer_usuario'])
    usuarios=ti.xcom_pull(task_ids=['extraer_usuario'], key="data")
    response=requests.get('https://randomuser.me/api')
    usuario=json.loads(response.text)
    print("Usuario:",usuario)
    if usuario is None:
   	 raise ValueError("usuario vacio")
    usuario = usuario['results'][0]
    usuario_procesado=pd.json_normalize({
   	 'nombre':usuario['name']['first'],
   	 'apellidos':usuario['name']['last'],
   	 'pais':usuario['location']['country'],
   	 'usuario':usuario['login']['username'],
   	 'password':usuario['login']['password'],
   	 'email':usuario['email']
    })
    usuario_procesado.to_csv('/tmp/usuario_procesado.csv',index=None,header=False)
    
# en schedule_interval le indicamos cada cuanto vamos a ejecutarlo
with DAG('procesar_usuario',schedule_interval='@daily', default_args = default_args, catchup=False) as dag:

    #definir tareas/operadores

    crear_tabla_sqlite = SqliteOperator(task_id="crear_tabla",
    sqlite_conn_id="db_sqlite",
    sql='''CREATE TABLE IF NOT EXISTS usuarios(nombre TEXT NOT NULL,
    apellidos TEXT NOT NULL,
    pais TEXT NOT NULL,
    usuario TEXT NOT NULL,
    password TEXT NOT NULL,
    email TEXT NOT NULL PRIMARY KEY);''')
    
    api_disponible = HttpSensor(task_id="api_disponible",
    http_conn_id="usuario_api",
    endpoint="api/")
    
    '''extraer_usuario = SimpleHttpOperator(task_id="extraer_usuario",
    http_conn_id="usuario_api",
    endpoint="api/",
    method="GET",
    response_filter=lambda response:json.loads(response.text),
    log_response=True)'''
    
    extraer_usuario = PythonOperator(
    	task_id='extraer_usuario',
    	python_callable=get_data
	)
    
    
    procesa_usuario = PythonOperator(task_id="procesa_usuario",python_callable=_procesa_usuario)
    
    almacenar_usuario = BashOperator(task_id="almacenar_usuario",
    bash_command='echo -e ".separator ","\n.import /tmp/usuario_procesado.csv usuarios" | sqlite3 /home/linux/airflow/airflow.db')
    crear_tabla_sqlite >> api_disponible >> extraer_usuario >> procesa_usuario >> almacenar_usuario
