import apache_beam as beam
from apache_beam.options.pipeline_options import PipelineOptions, GoogleCloudOptions, StandardOptions, WorkerOptions
from apache_beam.io.gcp.bigquery import WriteToBigQuery, BigQueryDisposition
import re

# Configurar opciones del pipeline
options = PipelineOptions()

google_cloud_options = options.view_as(GoogleCloudOptions)
google_cloud_options.project = 'tu-proyecto-id'
google_cloud_options.region = 'us-central1'
google_cloud_options.job_name = 'nombre-de-tu-job'
google_cloud_options.staging_location = 'gs://tu-bucket/staging'
google_cloud_options.temp_location = 'gs://tu-bucket/temp'

options.view_as(StandardOptions).runner = 'DataflowRunner'

# Opciones de trabajador
worker_options = options.view_as(WorkerOptions)
worker_options.num_workers = 5  # Número de trabajadores paralelos
worker_options.autoscaling_algorithm = 'THROUGHPUT_BASED'  # Autoscaling basado en el rendimiento

# Define el esquema de la tabla de BigQuery
table_schema = {
    'fields': [
        {'name': 'word', 'type': 'STRING', 'mode': 'REQUIRED'},
        {'name': 'count', 'type': 'INTEGER', 'mode': 'REQUIRED'}
    ]
}

# Función para tokenizar líneas en palabras
def tokenize(line):
    return re.findall(r'\b\w+\b', line.lower())

# Definir el pipeline
def run():
    with beam.Pipeline(options=options) as p:
        (p
         | 'Leer' >> beam.io.ReadFromText('gs://tu-bucket/input.txt')
         | 'Tokenizar' >> beam.FlatMap(tokenize)
         | 'Contar Palabras' >> beam.combiners.Count.PerElement()
         | 'Formato para BigQuery' >> beam.Map(lambda word_count: {'word': word_count[0], 'count': word_count[1]})
         | 'Escribir a BigQuery' >> WriteToBigQuery(
             table='tu-proyecto-id:tu_dataset.tu_tabla',
             schema=table_schema,
             write_disposition=BigQueryDisposition.WRITE_TRUNCATE,
             create_disposition=BigQueryDisposition.CREATE_IF_NEEDED
         )
        )

if __name__ == '__main__':
    run()
